@@ -28,7 +28,13 @@
 
 #include <assert.h>
 
-#ifdef WIN32
+#if defined(MINGW) || defined(__MINGW__) || defined(__MINGW32__) || defined(__MINGW64__)
+#ifndef WIN32
+#define WIN32
+#endif
+#endif
+
+#if defined(WIN32) || defined(__MINGW32__)
 #  include <winsock2.h>
 #  include <windows.h>
 #  include <windowsx.h>
@@ -76,19 +82,12 @@
 
 #include "sbuf.h"
 #include "utf8.h"
+#include "utils.h"                      // for gmtime_r
 
 #include <vector>
 #include <set>
 #include <map>
 
-namespace be13 {
-    struct hash_def {
-        hash_def():name(),func(){};
-        std::string name; // v3: (input) function to perform hashing with        
-        std::string (*func)(const uint8_t *buf,size_t bufsize); // v3: (input) function to perform hashing with
-    };
-};
-
 #include "feature_recorder.h"
 #include "feature_recorder_set.h"
 
@@ -625,7 +624,7 @@ private:
     static const int SCANNER_RECURSE_EXPAND = 0x020; // v3: recurses AND result is >= original size
     static const int SCANNER_WANTS_NGRAMS   = 0x040; // v3: Scanner gets buffers that are constant n-grams
     static const int SCANNER_FAST_FIND      = 0x080; // v3: This scanner is a very fast FIND scanner
-    static const int SCANNER_DEPTH_0        = 0x100; // v3: scanner only runs at detph 0 by default
+    static const int SCANNER_DEPTH_0        = 0x100; // v3: scanner only runs at depth 0 by default
     static const int CURRENT_SI_VERSION     = 4;     
 
     static const std::string flag_to_string(const int flag){
@@ -645,11 +644,10 @@ private:
      * Scanner histograms are added to 'histograms' by machinery.
      */
     struct scanner_config {
-        scanner_config():namevals(),debug(),hasher() /* ,histograms() */{};
+        scanner_config():namevals(),debug(){};
         virtual ~scanner_config(){}
         config_t  namevals;             // v3: (input) name=val map
         int       debug;                // v3: (input) current debug level
-        struct be13::hash_def  hasher;  // v3: (input) hasher to use
     };
 
     // never change the order or delete old fields, or else you will
@@ -774,15 +772,15 @@ class scanner_params {
      *** INSTANCE VARIABLES ***
      **************************/
 
-    const int sp_version;                /* version number of this structure */
-    const phase_t phase;                 /* v1: 0=startup, 1=normal, 2=shutdown (changed to phase_t in v1.3) */
-    const sbuf_t &sbuf;                 /* v1: what to scan / only valid in SCAN_PHASE */
-    class feature_recorder_set &fs;     /* v1: where to put the results / only valid in SCAN_PHASE */
-    const uint32_t depth;            /* v1: how far down are we? / only valid in SCAN_PHASE */
+    const int                   sp_version;                /* version number of this structure */
+    const phase_t               phase;                 /* v1: 0=startup, 1=normal, 2=shutdown (changed to phase_t in v1.3) */
+    const sbuf_t                &sbuf;                 /* v1: what to scan / only valid in SCAN_PHASE */
+    class feature_recorder_set  &fs;     /* v1: where to put the results / only valid in SCAN_PHASE */
+    const uint32_t              depth;            /* v1: how far down are we? / only valid in SCAN_PHASE */
 
-    PrintOptions  &print_options;    /* v1: how to print / NOT USED IN SCANNERS */
-    scanner_info  *info;             /* v2: set/get parameters on startup */
-    std::stringstream *sxml;         /* v3: on scanning and shutdown: CDATA added to XML stream (advanced feature) */
+    PrintOptions                &print_options;    /* v1: how to print / NOT USED IN SCANNERS */
+    scanner_info                *info;             /* v2: set/get parameters on startup, hasher */
+    std::stringstream           *sxml;         /* v3: on scanning and shutdown: CDATA added to XML stream (advanced feature) */
 };
 
 
@@ -885,6 +883,17 @@ inline std::string safe_utf16to8(std::wstring s){ // needs to be cleaned up
     return utf8_line;
 }
 
+inline std::wstring safe_utf8to16(std::string s){ // needs to be cleaned up
+    std::wstring utf16_line;
+    try {
+        utf8::utf8to16(s.begin(),s.end(),back_inserter(utf16_line));
+    } catch(utf8::invalid_utf8){
+        /* Exception thrown: bad UTF16 encoding */
+        utf16_line = L"";
+    }
+    return utf16_line;
+}
+
 // truncate string at the matching char
 inline void truncate_at(std::string &line, char ch) {
     size_t pos = line.find(ch);
@@ -909,10 +918,6 @@ inline int isxdigit(int c)
  * gmtime_r() is Linux-specific. You'll find a copy in util.cpp for Windows.
  */
 
-#ifndef HAVE_GMTIME_R
-void gmtime_r(time_t *t,struct tm *tm);
-#endif
-
 inline std::string microsoftDateToISODate(const uint64_t &time)
 {
     time_t tmp = (time / ONE_HUNDRED_NANO_SEC_TO_SECONDS) - SECONDS_BETWEEN_WIN32_EPOCH_AND_UNIX_EPOCH;
@@ -924,12 +929,24 @@ inline std::string microsoftDateToISODate(const uint64_t &time)
     return std::string(buf);
 }
 
+/* Convert Unix timestamp to ISO format */
+inline std::string unixTimeToISODate(const uint64_t &t)
+{
+    struct tm time_tm;
+    time_t tmp=t;
+    gmtime_r(&tmp, &time_tm);
+    char buf[256];
+    strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", &time_tm); // Zulu time
+    return std::string(buf);
+}
+
 /* Many internal windows and Linux structures require a valid printable name in ASCII */
 inline bool validASCIIName(const std::string &name)
 {
     for(size_t i = 0; i< name.size(); i++){
         if(((u_char)name[i]) & 0x80) return false; // high bit should not be set
         if(((u_char)name[i]) < ' ') return false;  // should not be control character
+        if(((u_char)name[i]) == 0x7f) return false; // DEL is not printable
     }
     return true;
 }
