@@ -3,7 +3,6 @@
 #include "config.h"
 #include "bulk_extractor_i.h"
 #include "unicode_escape.h"
-#include "beregex.h"
 #include "histogram.h"
 
 #include <unistd.h>
@@ -37,10 +36,47 @@ uint32_t feature_recorder::opt_max_feature_size=1024*1024;
 uint32_t feature_recorder::debug=0;
 
 
-void feature_recorder::banner_stamp(std::ostream &os,const std::string &header)
+/**
+ * Create a feature recorder object. Each recorder records a certain
+ * kind of feature.  Features are stored in a file. The filename is
+ * permutated based on the total number of threads and the current
+ * thread that's recording. Each thread records to a different file,
+ * and thus a different feature recorder, to avoid locking
+ * problems. 
+ *
+ * @param feature_recorder_set &fs - common information for all of the feature recorders
+ * @param name         - the name of the feature being recorded.
+ */
+
+feature_recorder::feature_recorder(class feature_recorder_set &fs_,
+                                   const std::string &name_):
+    flags(0),
+    name(name_),ignore_encoding(),ios(),bs(),
+    histogram_defs(),
+    fs(fs_),
+    count_(0),context_window_before(context_window_default),context_window_after(context_window_default),
+    Mf(),Mr(),mhistograms(),mhistogram_limit(),
+    stop_list_recorder(0),
+    file_number_(0),carve_mode(CARVE_ENCODED)
+{
+    //std::cerr << "feature_recorder(" << name << ") created\n";
+    open();                         // open if we are created
+}
+
+/* Don't have to delete the stop_list_recorder because it is in the
+ * feature_recorder_set and will be separately deleted.
+ */
+feature_recorder::~feature_recorder()
+{
+    if(ios.is_open()){
+        ios.close();
+    }
+}
+
+void feature_recorder::banner_stamp(std::ostream &os,const std::string &header) const
 {
     int banner_lines = 0;
-    if(banner_file!=""){
+    if(banner_file.size()>0){
         std::ifstream i(banner_file.c_str());
         if(i.is_open()){
             std::string line;
@@ -60,7 +96,7 @@ void feature_recorder::banner_stamp(std::ostream &os,const std::string &header)
     
     os << bulk_extractor_version_header;
     os << "# Feature-Recorder: " << name << "\n";
-    if(input_fname.size()) os << "# Filename: " << input_fname << "\n";
+    if(fs.get_input_fname().size()) os << "# Filename: " << fs.get_input_fname() << "\n";
     if(debug!=0){
         os << "# DEBUG: " << debug << " (";
         if(debug & DEBUG_PEDANTIC) os << " DEBUG_PEDANTIC ";
@@ -70,84 +106,67 @@ void feature_recorder::banner_stamp(std::ostream &os,const std::string &header)
 }
 
 
-/**
- * Create a feature recorder object. Each recorder records a certain
- * kind of feature.  Features are stored in a file. The filename is
- * permutated based on the total number of threads and the current
- * thread that's recording. Each thread records to a different file,
- * and thus a different feature recorder, to avoid locking
- * problems. 
- *
- * @param outdir_      - where the feature file is written
- * @param input_fname_ - the file (disk image) that these features were extracted from.
- *                     - We should probably have a callback function to annotate the feature file.
- * @param name         - the name of the feature being recorded.
- */
-
-feature_recorder::feature_recorder(class feature_recorder_set &fs_,
-                                   const std::string &outdir_,const std::string &input_fname_,const std::string &name_):
-    flags(0),
-    outdir(outdir_),input_fname(input_fname_),name(name_),ignore_encoding(),ios(),
-    histogram_defs(),
-    fs(fs_),
-    count_(0),context_window_before(context_window_default),context_window_after(context_window_default),
-    Mf(),Mr(),mhistogram(),
-    stop_list_recorder(0),
-    file_number_(0),carve_mode(CARVE_ENCODED)
-{
-}
-
-/* Don't have to delete the stop_list_recorder because it is in the
- * feature_recorder_set and will be separately deleted.
- */
-feature_recorder::~feature_recorder()
-{
-    if(ios.is_open()){
-        ios.close();
-    }
-}
-
 
 /**
  * Return the filename with a counter
  */
 std::string feature_recorder::fname_counter(std::string suffix) const
 {
-    return outdir + "/" + this->name + (suffix.size()>0 ? (std::string("_") + suffix) : "") + ".txt";
+    return fs.get_outdir() + "/" + this->name + (suffix.size()>0 ? (std::string("_") + suffix) : "") + ".txt";
 }
 
 
+const std::string &feature_recorder::get_outdir() const 
+{
+    return fs.get_outdir();
+}
+
 /**
  * open a feature recorder file in the specified output directory.
+ * Called by create_name(). Not clear why it isn't called when created.
  */
 
 void feature_recorder::open()
 { 
-    std::string fname = fname_counter("");
-    ios.open(fname.c_str(),std::ios_base::in|std::ios_base::out|std::ios_base::ate);
-    if(ios.is_open()){                  // opened existing stream
-        ios.seekg(0L,std::ios_base::end);
-        while(ios.is_open()){
-            /* Get current position */
-            if(int(ios.tellg())==0){            // at beginning of file; stamp and return
-                ios.seekp(0L,std::ios_base::beg);    // be sure we are at the beginning of the file
-                return;
-            }
-            ios.seekg(-1,std::ios_base::cur); // backup to once less than the end of the file
-            if (ios.peek()=='\n'){           // we are finally on the \n
-                ios.seekg(1L,std::ios_base::cur); // move the getting one forward
-                ios.seekp(ios.tellg(),std::ios_base::beg); // put the putter at the getter location
-                count_ = 1;                            // greater than zero
-                return;
+    if (fs.flag_set(feature_recorder_set::SET_DISABLED)) return;        // feature recorder set is disabled
+
+    /* write to a database? Create tables if necessary and create a prepared statement */
+    if (fs.flag_set(feature_recorder_set::ENABLE_SQLITE3_RECORDERS)) {  
+        char buf[1024];
+        fs.db_create_table(name);
+        snprintf(buf,sizeof(buf),db_insert_stmt,name.c_str());
+        bs = new besql_stmt(fs.db3,buf);
+    }
+
+    /* Write to a file? Open the file and seek to the last line if it exist, otherwise just open database */
+    if (fs.flag_notset(feature_recorder_set::DISABLE_FILE_RECORDERS)){
+        /* Open the file recorder */
+        std::string fname = fname_counter("");
+        ios.open(fname.c_str(),std::ios_base::in|std::ios_base::out|std::ios_base::ate);
+        if(ios.is_open()){                  // opened existing stream
+            ios.seekg(0L,std::ios_base::end);
+            while(ios.is_open()){
+                /* Get current position */
+                if(int(ios.tellg())==0){            // at beginning of file; stamp and return
+                    ios.seekp(0L,std::ios_base::beg);    // be sure we are at the beginning of the file
+                    return;
+                }
+                ios.seekg(-1,std::ios_base::cur); // backup to once less than the end of the file
+                if (ios.peek()=='\n'){           // we are finally on the \n
+                    ios.seekg(1L,std::ios_base::cur); // move the getting one forward
+                    ios.seekp(ios.tellg(),std::ios_base::beg); // put the putter at the getter location
+                    count_ = 1;                            // greater than zero
+                    return;
+                }
             }
         }
-    }
-    // Just open the stream for output
-    ios.open(fname.c_str(),std::ios_base::out);
-    if(!ios.is_open()){
-        std::cerr << "*** feature_recorder::open CANNOT OPEN FEATURE FILE FOR WRITING "
-             << fname << ":" << strerror(errno) << "\n";
-        exit(1);
+        // Just open the stream for output
+        ios.open(fname.c_str(),std::ios_base::out);
+        if(!ios.is_open()){
+            std::cerr << "*** feature_recorder::open CANNOT OPEN FEATURE FILE FOR WRITING "
+                      << fname << ":" << strerror(errno) << "\n";
+            exit(1);
+        }
     }
 }
 
@@ -177,9 +196,24 @@ const std::string feature_recorder::bulk_extractor_version_header("# " PACKAGE_N
 
 static inline int hexval(char ch)
 {
-    if(ch>='0' && ch<='9') return ch-'0';
-    if(ch>='a' && ch<='f') return ch-'a'+10;
-    if(ch>='A' && ch<='F') return ch-'a'+10;
+    switch (ch) {
+    case '0': return 0;
+    case '1': return 1;
+    case '2': return 2;
+    case '3': return 3;
+    case '4': return 4;
+    case '5': return 5;
+    case '6': return 6;
+    case '7': return 7;
+    case '8': return 8;
+    case '9': return 9;
+    case 'a': case 'A': return 10;
+    case 'b': case 'B': return 11;
+    case 'c': case 'C': return 12;
+    case 'd': case 'D': return 13;
+    case 'e': case 'E': return 14;
+    case 'f': case 'F': return 15;
+    }
     return 0;
 }
 
@@ -202,7 +236,7 @@ std::string feature_recorder::unquote_string(const std::string &s)
         }
         /* Look for hex coding */
         if(i+3<len && s[i]=='\\' && s[i+1]=='x' && isxdigit(s[i+2]) && isxdigit(s[i+3])){
-            uint8_t code = (hexval(s[i+2])<<4) | hexval(s[i+3]);
+            uint8_t code = (hexval(s[i+2])*16) | hexval(s[i+3]);
             out.push_back(code);
             i += 3;                     // skip over the digits
             continue;
@@ -230,20 +264,26 @@ void feature_recorder::set_flag(uint32_t flags_)
 {
     MAINTHREAD();
     flags|=flags_;
-
-    if((flags & FLAG_MEM_HISTOGRAM) && mhistogram==0){
-        /* Create the in-memory histogram */
-        mhistogram = new mhistogram_t();
-    }
 }
 
 void feature_recorder::unset_flag(uint32_t flags_)
 {
     MAINTHREAD();
     flags &= (~flags_);
+}
 
-    if((flags & FLAG_MEM_HISTOGRAM) && mhistogram!=0){
-        assert(0);                      // we can't turn it off at the moment
+void feature_recorder::set_memhist_limit(int64_t limit_)
+{
+    MAINTHREAD();
+    mhistogram_limit = limit_;
+}
+
+
+// add a memory histogram; assume the position in the mhistograms is stable
+void feature_recorder::enable_memory_histograms()
+{
+    for(histogram_defs_t::const_iterator it=histogram_defs.begin();it!=histogram_defs.end();it++){
+        mhistograms[*it] = new mhistogram_t(); 
     }
 }
 
@@ -253,11 +293,12 @@ void feature_recorder::unset_flag(uint32_t flags_)
  */
 
 /* dump_callback_test is a simple callback that just prints to stderr. It's for testing */
-void feature_recorder::dump_callback_test(void *user,const feature_recorder &fr,
+int feature_recorder::dump_callback_test(void *user,const feature_recorder &fr,
                                           const std::string &str,const uint64_t &count)
 {
     (void)user;
     std::cerr << "dump_cb: user=" << user << " " << str << ": " << count << "\n";
+    return 0;
 }
 
 /* Make a histogram. If a callback is provided, send the output there. */
@@ -266,29 +307,44 @@ class mhistogram_callback {
     mhistogram_callback &operator=(const mhistogram_callback &);
 public:
     mhistogram_callback(void *user_,
-                             feature_recorder::dump_callback_t *cb_,
-                             const feature_recorder &fr_):user(user_),cb(cb_),fr(fr_){}
+                        feature_recorder::dump_callback_t *cb_,
+                        const histogram_def &def_,
+                        const feature_recorder &fr_,
+                        uint64_t limit_):user(user_),cb(cb_),def(def_),fr(fr_),callback_count(0),limit(limit_){}
     void *user;
     feature_recorder::dump_callback_t *cb;
+    const histogram_def &def;
     const feature_recorder &fr;
-    static void callback(void *user,const std::string &str,const uint64_t &count) {
-        mhistogram_callback &mcbo = *(mhistogram_callback *)(user);
-        mcbo.cb(mcbo.user,mcbo.fr,str,count);
+    uint64_t callback_count;
+    uint64_t limit;
+    int do_callback(const std::string &str,const uint64_t &tally){
+        (*cb)(user,fr,def,str,tally);
+        if(limit && ++callback_count >= limit) return -1;
+        return 0;
+    }
+    static int callback(void *ptr,const std::string &str,const uint64_t &tally) {
+        return ((mhistogram_callback *)(ptr))->do_callback(str,tally);
     }
 };
 
-void feature_recorder::dump_histogram(const class histogram_def &def,void *user,feature_recorder::dump_callback_t cb) 
+/****************************************************************
+ *** PHASE HISTOGRAM (formerly phase 3): Create the histograms
+ ****************************************************************/
+
+/**
+ * We now have three kinds of histograms:
+ * 1 - Traditional post-processing histograms specified by the histogram library
+ *   1a - feature-file based traditional ones
+ *   1b - SQL-based traditional ones.
+ * 2 - In-memory histograms (used primarily by beapi)
+ */
+
+
+/** Dump a specific histogram */
+void feature_recorder::dump_histogram_file(const histogram_def &def,void *user,feature_recorder::dump_callback_t cb) const
 {
-    if(mhistogram){
-        assert(cb!=0);
-        mhistogram_callback mcbo(user,cb,*this);
-        mhistogram->dump_sorted(static_cast<void *>(&mcbo),mhistogram_callback::callback);
-        return;
-    }
-
-    beregex reg(def.pattern,REG_EXTENDED);
+    /* This is a file based histogram. We will be reading from one file and writing to another */
     std::string ifname = fname_counter("");  // source of features
-
     std::ifstream f(ifname.c_str());
     if(!f.is_open()){
         std::cerr << "Cannot open histogram input file: " << ifname << "\n";
@@ -323,7 +379,7 @@ void feature_recorder::dump_histogram(const class histogram_def &def,void *user,
                 /** If there is a pattern to use to prune down the feature, use it */
                 if(def.pattern.size()){
                     std::string new_feature = feature;
-                    if(!reg.search(feature,&new_feature,0,0)){
+                    if(!def.reg.search(feature,&new_feature,0,0)){
                         // no search match; avoid this feature
                         continue;               
                     }
@@ -342,14 +398,14 @@ void feature_recorder::dump_histogram(const class histogram_def &def,void *user,
                       << name << "\n";
         }
             
-        /* Output what we have */
+        /* Output what we have to a new file ofname */
         std::stringstream real_suffix;
 
         real_suffix << def.suffix;
         if(histogram_counter>0) real_suffix << histogram_counter;
         std::string ofname = fname_counter(real_suffix.str()); // histogram name
         std::ofstream o;
-        o.open(ofname.c_str());
+        o.open(ofname.c_str());         // open the file
         if(!o.is_open()){
             std::cerr << "Cannot open histogram output file: " << ofname << "\n";
             return;
@@ -361,6 +417,9 @@ void feature_recorder::dump_histogram(const class histogram_def &def,void *user,
             o << *fr;                   // sends the entire histogram
         }
 
+        for(size_t i = 0;i<fr->size();i++){
+            delete fr->at(i);
+        }
         delete fr;
         o.close();
 
@@ -373,39 +432,52 @@ void feature_recorder::dump_histogram(const class histogram_def &def,void *user,
 }
 
 
-void feature_recorder::add_histogram(const histogram_def &def)
+void feature_recorder::dump_histogram(const histogram_def &def,void *user,feature_recorder::dump_callback_t cb) const
 {
-    histogram_defs.insert(def);
+    /* Inform that we are dumping this histogram */
+    if(cb) cb(user,*this,def,"",0); 
+
+    /* If this is a memory histogram, dump it and return */
+    mhistograms_t::const_iterator it = mhistograms.find(def);
+    if(it!=mhistograms.end()){
+        assert(cb!=0);
+        mhistogram_callback mcbo(user,cb,def,*this,mhistogram_limit);
+        it->second->dump_sorted(static_cast<void *>(&mcbo),mhistogram_callback::callback);
+        return;
+    }
+
+    if (fs.flag_set(feature_recorder_set::ENABLE_SQLITE3_RECORDERS)) {
+        dump_histogram_db(def,user,cb);
+    }
+    
+
+    if (fs.flag_notset(feature_recorder_set::DISABLE_FILE_RECORDERS)) {
+        dump_histogram_file(def,user,cb);
+    }
 }
 
-/* Dump all of our histograms */
+
+/* Dump all of this feature recorders histograms */
 
 
 void feature_recorder::dump_histograms(void *user,feature_recorder::dump_callback_t cb,
-                                           feature_recorder_set::xml_notifier_t xml_error_notifier)
+                                           feature_recorder_set::xml_notifier_t xml_error_notifier) const
 {
-    /* See if we have an in-memory histograms */
-    if(flag_set(feature_recorder::FLAG_MEM_HISTOGRAM)){
-        std::cerr << "***************** " << name << " has a memory histogram\n";
-        histogram_def d("","","",0);            // empty
-        dump_histogram(d,user,cb);
-    }
-       
-    /* Loop through all the histograms */
+    /* If we are recording features to SQL and we have a histogram defintion
+     * for this feature recorder, we need to create a base histogram first,
+     * then we can create the extracted histograms if they are presented.
+     */
+
+
+    /* Loop through all the histograms and dump each one.
+     * This now works for both memory histograms and non-memory histograms.
+     */
     for(histogram_defs_t::const_iterator it = histogram_defs.begin();it!=histogram_defs.end();it++){
-        std::cout << std::string(" ") << name << " " << (*it).suffix + "...\n";
-        std::cout.flush();
         try {
-            if(flag_set(feature_recorder::FLAG_MEM_HISTOGRAM)){
-                std::cerr << name << " cannot have both a regular histogram and a memory histogram\n";
-            } else {
-                dump_histogram((*it),user,cb);
-            }
+            dump_histogram((*it),user,cb);
         }
         catch (const std::exception &e) {
-            std::cerr << "ERROR: " ;
-            std::cerr.flush();
-            std::cerr << e.what() << " computing histogram " << name << "\n";
+            std::cerr << "ERROR: histogram " << name << ": " << e.what() << "\n";
             if(xml_error_notifier){
                 std::string error = std::string("<error function='phase3' histogram='")
                     + name + std::string("</error>");
@@ -416,6 +488,12 @@ void feature_recorder::dump_histograms(void *user,feature_recorder::dump_callbac
 }
 
 
+void feature_recorder::add_histogram(const histogram_def &def)
+{
+    histogram_defs.insert(def);
+}
+
+
 
 /****************************************************************
  *** WRITING SUPPORT
@@ -432,10 +510,15 @@ void feature_recorder::write(const std::string &str)
             std::cerr << "******************************************\n";
             std::cerr << "feature recorder: " << name << "\n";
             std::cerr << "invalid UTF-8 in write: " << str << "\n";
-            std::cerr << "ABORT\n";
-            exit(1);
+            assert(0);
         }
     }
+
+    /* This is where the writing happens. Lock the output and write */
+    if (fs.flag_set(feature_recorder_set::DISABLE_FILE_RECORDERS)) {
+        return;
+    }
+
     cppmutex::lock lock(Mf);
     if(ios.is_open()){
         if(count_==0){
@@ -468,14 +551,25 @@ void feature_recorder::printf(const char *fmt, ...)
 
 /**
  * Combine the pos0, feature and context into a single line and write it to the feature file.
+ *
+ * @param feature - The feature, which is valid UTF8 (but may not be exactly the bytes on the disk)
+ * @param context - The context, which is valid UTF8 (but may not be exactly the bytes on the disk)
+ *
+ * Interlocking is done in write().
  */
 
 void feature_recorder::write0(const pos0_t &pos0,const std::string &feature,const std::string &context)
 {
-    std::stringstream ss;
-    ss << pos0.shift(feature_recorder::offset_add).str() << '\t' << feature;
-    if(flag_notset(FLAG_NO_CONTEXT) && (context.size()>0)) ss << '\t' << context;
-    this->write(ss.str());
+    if ( fs.flag_set(feature_recorder_set::ENABLE_SQLITE3_RECORDERS ) &&
+         this->flag_notset(feature_recorder::FLAG_NO_FEATURES_SQL) ) {
+        db_write0( pos0, feature, context);
+    }
+    if ( fs.flag_notset(feature_recorder_set::DISABLE_FILE_RECORDERS )) {
+        std::stringstream ss;
+        ss << pos0.shift( feature_recorder::offset_add).str() << '\t' << feature;
+        if (flag_notset( FLAG_NO_CONTEXT ) && ( context.size()>0 )) ss << '\t' << context;
+        this->write( ss.str() );
+    }
 }
 
 
@@ -484,20 +578,8 @@ void feature_recorder::write0(const pos0_t &pos0,const std::string &feature,cons
  * processes the stop list
  */
 
-void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,const std::string &context_)
+void feature_recorder::quote_if_necessary(std::string &feature,std::string &context)
 {
-    if(flags & FLAG_DISABLED) return;           // disabled
-    if(debug & DEBUG_PEDANTIC){
-        if(feature_.size() > opt_max_feature_size){
-            std::cerr << "feature_recorder::write : feature_.size()=" << feature_.size() << "\n";
-            assert(0);
-        }
-        if(context_.size() > opt_max_context_size){
-            std::cerr << "feature_recorder::write : context_.size()=" << context_.size() << "\n";
-            assert(0);
-        }
-    }
-
     /* By default quote string that is not UTF-8, and quote backslashes. */
     bool escape_bad_utf8  = true;
     bool escape_backslash = true;
@@ -512,15 +594,38 @@ void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,cons
         escape_backslash = false;
     }
 
-    std::string feature = validateOrEscapeUTF8(feature_, escape_bad_utf8,escape_backslash);
-
-    std::string context;
+    feature = validateOrEscapeUTF8(feature, escape_bad_utf8,escape_backslash);
+    if(feature.size() > opt_max_feature_size) feature.resize(opt_max_feature_size);
     if(flag_notset(FLAG_NO_CONTEXT)){
-        context = validateOrEscapeUTF8(context_,escape_bad_utf8,escape_backslash);
+        context = validateOrEscapeUTF8(context,escape_bad_utf8,escape_backslash);
+        if(context.size() > opt_max_context_size) context.resize(opt_max_context_size);
+    }
+}
+
+/**
+ * write() is the main entry point for writing a feature at a given position with context.
+ * write() checks the stoplist and escapes non-UTF8 characters, then calls write0().
+ */
+void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,const std::string &context_)
+{
+    if(flags & FLAG_DISABLED) return;           // disabled
+    if(debug & DEBUG_PEDANTIC){
+        if(feature_.size() > opt_max_feature_size){
+            std::cerr << "feature_recorder::write : feature_.size()=" << feature_.size() << "\n";
+            assert(0);
+        }
+        if(context_.size() > opt_max_context_size){
+            std::cerr << "feature_recorder::write : context_.size()=" << context_.size() << "\n";
+            assert(0);
+        }
     }
 
-    if(feature.size() > opt_max_feature_size) feature.resize(opt_max_feature_size);
-    if(context.size() > opt_max_context_size) context.resize(opt_max_context_size);
+    std::string feature = feature_;
+    std::string context = flag_set(FLAG_NO_CONTEXT) ? "" : context_;
+    std::string *feature_utf8 = HistogramMaker::make_utf8(feature); // a utf8 feature
+
+    quote_if_necessary(feature,context);
+
     if(feature.size()==0){
         std::cerr << "zero length feature at " << pos0 << "\n";
         if(debug & DEBUG_PEDANTIC) assert(0);
@@ -546,8 +651,9 @@ void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,cons
      */
     if(flag_notset(FLAG_NO_STOPLIST) && stop_list_recorder){          
         if(fs.stop_list
-           && fs.stop_list->check_feature_context(feature,context)){
+           && fs.stop_list->check_feature_context(*feature_utf8,context)){
             stop_list_recorder->write(pos0,feature,context);
+            delete feature_utf8;
             return;
         }
     }
@@ -557,10 +663,9 @@ void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,cons
      */
     if(flag_notset(FLAG_NO_ALERTLIST)
        && fs.alert_list
-       && fs.alert_list->check_feature_context(feature,context)){
-        std::string alert_fn = outdir + "/ALERTS_found.txt";
-
-        cppmutex::lock lock(Mr);                // notce we are locking the redlist
+       && fs.alert_list->check_feature_context(*feature_utf8,context)){
+        std::string alert_fn = fs.get_outdir() + "/ALERTS_found.txt";
+        cppmutex::lock lock(Mr);                // notice we are locking the alert list
         std::ofstream rf(alert_fn.c_str(),std::ios_base::app);
         if(rf.is_open()){
             rf << pos0.shift(feature_recorder::offset_add).str() << '\t' << feature << '\t' << "\n";
@@ -568,14 +673,27 @@ void feature_recorder::write(const pos0_t &pos0,const std::string &feature_,cons
     }
 
     /* Support in-memory histograms */
-    if(mhistogram){
-        mhistogram->add(feature,1);
+    for(mhistograms_t::iterator it = mhistograms.begin(); it!=mhistograms.end();it++){
+        const histogram_def &def = it->first;
+        mhistogram_t *m = it->second;
+        std::string new_feature = *feature_utf8;
+        if(def.require.size()==0 || new_feature.find_first_of(def.require)!=std::string::npos){
+            /* If there is a pattern to use, use it */
+            if(def.pattern.size()){
+                if(!def.reg.search(new_feature,&new_feature,0,0)){
+                    // no search match; avoid this feature
+                    new_feature = "";
+                }
+            }
+            if(new_feature.size()) m->add(new_feature,1);
+        }
     }
 
     /* Finally write out the feature and the context */
     if(flag_notset(FLAG_NO_FEATURES)){
         this->write0(pos0,feature,context);
     }
+    delete feature_utf8;
 }
 
 /**
@@ -653,6 +771,7 @@ std::string replace(const std::string &src,char f,char t)
  ****************************************************************
  *
  * Carving support.
+ * 2014-04-24 - $ is no longer valid either
  * 2013-08-29 - replace invalid characters in filenames
  * 2013-07-30 - automatically bin directories
  * 2013-06-08 - filenames are the forensic path.
@@ -667,7 +786,7 @@ std::string valid_dosname(std::string in)
            || ch=='"' || ch=='*' || ch=='+' || ch==','
            || ch=='/' || ch==':' || ch==';' || ch=='<'
            || ch=='=' || ch=='>' || ch=='?' || ch=='\\'
-           || ch=='[' || ch==']' || ch=='|'){
+           || ch=='[' || ch==']' || ch=='|' || ch=='$' ){
             out.push_back('_');
         } else {
             out.push_back(ch);
@@ -677,17 +796,22 @@ std::string valid_dosname(std::string in)
 }
         
 
+//const feature_recorder::hash_def &feature_recorder::hasher()
+//{
+//    return fs.hasher;
+//}
+
+
+
 #include <iomanip>
 /**
  * @param sbuf   - the buffer to carve
  * @param pos    - offset in the buffer to carve
  * @param len    - how many bytes to carve
- * @param hasher - to compute the hash of the carved object.
  *
  */
 std::string feature_recorder::carve(const sbuf_t &sbuf,size_t pos,size_t len,
-                                    const std::string &ext,
-                                    const be13::hash_def &hasher)
+                                    const std::string &ext)
 {
     if(flags & FLAG_DISABLED) return std::string();           // disabled
 
@@ -695,11 +819,7 @@ std::string feature_recorder::carve(const sbuf_t &sbuf,size_t pos,size_t len,
     if(pos >= sbuf.pagesize && pos < sbuf.bufsize){
         return std::string();
     }
-
-    if(pos >= sbuf.bufsize){    /* Sanity checks */
-        std::cerr << "*** carve: WRITE OUTSIDE BUFFER.  pos=" << pos << " sbuf=" << sbuf << "\n";
-        return std::string();
-    }
+    assert(pos < sbuf.bufsize);
 
     /* Carve to a file depending on the carving mode.  The purpose
      * of CARVE_ENCODED is to allow us to carve JPEGs when they are
@@ -734,21 +854,22 @@ std::string feature_recorder::carve(const sbuf_t &sbuf,size_t pos,size_t len,
      */
 
     uint64_t this_file_number = file_number_add(1);
-    std::string dirname1 = outdir + "/" + name;
+    std::string dirname1 = fs.get_outdir() + "/" + name;
     std::stringstream ss;
 
     ss << dirname1 << "/" << std::setw(3) << std::setfill('0') << (this_file_number / 1000);
 
     std::string dirname2 = ss.str(); 
-    std::string fname    = dirname2 + std::string("/") + valid_dosname(sbuf.pos0.str() + ext);
-    std::string carved_hash_hexvalue = (*hasher.func)(sbuf.buf,sbuf.bufsize);
+    std::string fname         = dirname2 + std::string("/") + valid_dosname(sbuf.pos0.str() + ext);
+    std::string fname_feature = fname.substr(fs.get_outdir().size()+1); 
+    std::string carved_hash_hexvalue = (*fs.hasher.func)(sbuf.buf,sbuf.bufsize);
 
     /* Record what was found in the feature file.
      */
     ss.str(std::string()); // clear the stringstream
     ss << "<fileobject><filename>" << fname << "</filename><filesize>" << len << "</filesize>"
-       << "<hashdigest type='" << hasher.name << "'>" << carved_hash_hexvalue << "</hashdigest></fileobject>";
-    this->write(sbuf.pos0+len,fname,ss.str());
+       << "<hashdigest type='" << fs.hasher.name << "'>" << carved_hash_hexvalue << "</hashdigest></fileobject>";
+    this->write(sbuf.pos0+len,fname_feature,ss.str());
     
     /* Make the directory if it doesn't exist.  */
     if (access(dirname2.c_str(),R_OK)!=0){
