@@ -46,35 +46,6 @@ bool scanner_commands_processed = false;
  *** misc support
  ****************************************************************/
 
-#ifndef HAVE_ERR
-#include <stdarg.h>
-// noreturn attribute to avoid warning with GCC on Linux
-static void err(int eval,const char *fmt,...) __attribute__ ((noreturn));
-static void err(int eval,const char *fmt,...)
-{
-    va_list ap;
-    va_start(ap,fmt);
-    vfprintf(stderr,fmt,ap);
-    va_end(ap);
-    fprintf(stderr,": %s\n",strerror(errno));
-    exit(eval);
-}
-#endif
-
-#ifndef HAVE_ERRX
-#include <stdarg.h>
-// noreturn attribute to avoid warning with GCC on Linux
-static void errx(int eval,const char *fmt,...) __attribute__ ((noreturn));
-static void errx(int eval,const char *fmt,...)
-{
-    va_list ap;
-    va_start(ap,fmt);
-    vfprintf(stderr,fmt,ap);
-    fprintf(stderr,"%s\n",strerror(errno));
-    va_end(ap);
-    exit(eval);
-}
-#endif
 
 /****************************************************************
  *** SCANNER PLUG-IN SYSTEM
@@ -158,7 +129,7 @@ void be13::plugin::set_scanner_enabled_all(bool enable)
  * As part of scanner loading:
  * - pass configuration to the scanner
  * - feature files that the scanner requires
- * - Histograms that the scanner makes
+ * - Histograms that the scanner makes (see feature_recorder_set)
  * This is called before scanners are enabled or disabled, so the pcap handlers
  * need to be set afterwards
  */
@@ -173,7 +144,9 @@ void be13::plugin::load_scanner(scanner_t scanner,const scanner_info::scanner_co
      * we use static values so that the sbuf is not constantly being created and destroyed.
      */
     static const sbuf_t sbuf;
-    static feature_recorder_set fs(feature_recorder_set::SET_DISABLED); // dummy
+    static feature_recorder_set fs(feature_recorder_set::SET_DISABLED,feature_recorder_set::null_hasher,
+                                   feature_recorder_set::NO_INPUT,
+                                   feature_recorder_set::NO_OUTDIR); // dummy
 
     //
     // Each scanner's params are stored in a scanner_def object that
@@ -207,7 +180,8 @@ void be13::plugin::load_scanner_file(std::string fn,const scanner_info::scanner_
     /* Figure out the function name */
     size_t extloc = fn.rfind('.');
     if(extloc==std::string::npos){
-        errx(1,"Cannot find '.' in %s",fn.c_str());
+        fprintf(stderr,"Cannot find '.' in %s",fn.c_str());
+        exit(1);
     }
     std::string func_name = fn.substr(0,extloc);
     size_t slashloc = func_name.rfind('/');
@@ -215,26 +189,36 @@ void be13::plugin::load_scanner_file(std::string fn,const scanner_info::scanner_
     slashloc = func_name.rfind('\\');
     if(slashloc!=std::string::npos) func_name = func_name.substr(slashloc+1);
 
-    std::cout << "Loading: " << fn << " (" << func_name << ")\n";
+    if(debug) std::cout << "Loading: " << fn << " (" << func_name << ")\n";
     scanner_t *scanner = 0;
 #if defined(HAVE_DLOPEN)
     void *lib=dlopen(fn.c_str(), RTLD_LAZY);
 
     if(lib==0){
-        errx(1,"dlopen: %s\n",dlerror());
+        fprintf(stderr,"dlopen: %s\n",dlerror());
+        exit(1);
     }
 
     /* Resolve the symbol */
     scanner = (scanner_t *)dlsym(lib, func_name.c_str());
 
-    if(scanner==0) errx(1,"dlsym: %s\n",dlerror());
+    if(scanner==0){
+        fprintf(stderr,"dlsym: %s\n",dlerror());
+        exit(1);
+    }
 #elif defined(HAVE_LOADLIBRARY)
     /* Use Win32 LoadLibrary function */
     /* See http://msdn.microsoft.com/en-us/library/ms686944(v=vs.85).aspx */
     HINSTANCE hinstLib = LoadLibrary(TEXT(fn.c_str()));
-    if(hinstLib==0) errx(1,"LoadLibrary(%s) failed",fn.c_str());
+    if(hinstLib==0){
+        fprintf(stderr,"LoadLibrary(%s) failed",fn.c_str());
+        exit(1);
+    }
     scanner = (scanner_t *)GetProcAddress(hinstLib,func_name.c_str());
-    if(scanner==0) errx(1,"GetProcAddress(%s) failed",func_name.c_str());
+    if(scanner==0){
+        fprintf(stderr,"GetProcAddress(%s) failed",func_name.c_str());
+        exit(1);
+    }
 #else
     std::cout << "  ERROR: Support for loadable libraries not enabled\n";
     return;
@@ -253,7 +237,8 @@ void be13::plugin::load_scanner_directory(const std::string &dirname,const scann
 {
     DIR *dirp = opendir(dirname.c_str());
     if(dirp==0){
-        err(1,"Cannot open directory %s:",dirname.c_str());
+        fprintf(stderr,"Cannot open directory %s:",dirname.c_str());
+        exit(1);
     }
     struct dirent *dp;
     while ((dp = readdir(dirp)) != NULL){
@@ -420,18 +405,12 @@ void be13::plugin::phase_shutdown(feature_recorder_set &fs,std::stringstream *sx
     }
 }
 
-/****************************************************************
- *** PHASE HISTOGRAM (formerly phase 3): Create the histograms
- ****************************************************************/
+/************************************
+ *** HELP and  option processing  ***
+ ************************************/
 
-/**
- * Note currently we have two kinds of histograms:
- * post-processing histograms specified by the histogram library, and in-memory histograms.
- * that are really only used by scan_bulk.
- */
-
-/* option processing */
 /* Get the config and build the help strings at the same time! */
+
 std::stringstream scanner_info::helpstream;
 void scanner_info::get_config(const scanner_info::config_t &c,
                               const std::string &n,std::string *val,const std::string &help)
@@ -598,6 +577,8 @@ void be13::plugin::process_sbuf(const class scanner_params &sp)
     const pos0_t &pos0 = sp.sbuf.pos0;
     class feature_recorder_set &fs = sp.fs;
 
+    fs.heartbeat();                     // note that we are alive
+
     {
         /* note the maximum depth that we've seen */
         cppmutex::lock lock(max_depth_seenM);
