@@ -20,7 +20,7 @@
 
 #include "config.h"
 
-#ifdef WIN32
+#ifdef HAVE_WINSOCK2_H
 #include <winsock2.h>
 #endif
 
@@ -335,7 +335,7 @@ extern "C" {
         int size = vsnprintf(buf,sizeof(buf),fmt,ap);
         if(size<0) return size;
         /* Now allocate the memory */
-        *ret = (char *)strdup(buf);
+        *ret = strcpy((char *) malloc(strlen(buf)+1), buf);
         return size;
     }
 }
@@ -652,12 +652,8 @@ void dfxml_writer::xmlout(const string &tag,const string &value,const string &at
 #include <hashdb.hpp>
 #endif
 
-#ifdef HAVE_ZMQ_H
-#include <zmq.h>
-#endif
-
 #ifdef HAVE_AFFLIB_AFFLIB_H
-#pragma GCC diagnostic ignored "-Wreserved-user-defined-literal"               // required for C11
+//#pragma GCC diagnostic ignored "-Wreserved-user-defined-literal"               // required for C11
 #include <afflib/afflib.h>
 #endif
 
@@ -723,15 +719,6 @@ void dfxml_writer::add_DFXML_build_environment()
 #ifdef SQLITE_VERSION
     xmlout("library", "", "name=\"sqlite\" version=\"" SQLITE_VERSION "\" source_id=\"" SQLITE_SOURCE_ID "\"",false);
 #endif
-#ifdef HAVE_ZMQ_VERSION
-    {
-        int zmq_major, zmq_minor, zmq_patch;
-        zmq_version (&zmq_major, &zmq_minor, &zmq_patch);
-        stringstream zmq_ss;
-        zmq_ss << zmq_major << "." << zmq_minor << "." << zmq_patch;
-        xmlout("library", "", std::string("name=\"zmq\" version=\"") + zmq_ss.str() + "\"",false);
-    }
-#endif
 #ifdef HAVE_GNUEXIF
     // gnuexif does not have a programmatically obtainable version.
     xmlout("library","","name=\"gnuexif\" version=\"?\"",false);
