@@ -19,19 +19,69 @@
 #include <sstream>
 #include <vector>
 
-
-
 /* static */ uint32_t tcpdemux::max_saved_flows = 100;
 /* static */ uint32_t tcpdemux::tcp_timeout = 0;
 
-tcpdemux::tcpdemux():outdir("."),flow_counter(0),packet_counter(0),
-		     xreport(0),pwriter(0),max_open_flows(),max_fds(get_max_fds()-NUM_RESERVED_FDS),
-                     flow_map(),open_flows(),saved_flow_map(),
-		     saved_flows(),start_new_connections(false),opt(),fs()
-		     
+tcpdemux::tcpdemux():
+#ifdef HAVE_SQLITE3
+    db(),insert_flow(),
+#endif
+    outdir("."),flow_counter(0),packet_counter(0),
+    xreport(0),pwriter(0),max_open_flows(),max_fds(get_max_fds()-NUM_RESERVED_FDS),
+    flow_map(),open_flows(),saved_flow_map(),
+    saved_flows(),start_new_connections(false),opt(),fs()
 {
 }
 
+void tcpdemux::openDB()
+{
+#ifdef HAVE_SQLITE3
+    int rc = sqlite3_open("test.db", &db);
+    if( rc ){
+        fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
+        db = 0;
+    }
+    /* Create SQL statement */
+    const char *sql = "CREATE TABLE connections ("
+        "starttime TEXT NOT NULL,"
+        "endtime TEXT NOT NULL,"
+        "src_ipn TEXT,"  
+        "dst_ipn TEXT,"
+        "mac_daddr TEXT,"
+        "mac_saddr TEXT,"
+        "packets INTEGER,"
+        "srcport INTEGER,"
+        "dstport INTEGER,"
+        "hashdigest_md5 TEXT);";
+
+    /* Execute SQL statement */
+    rc = sqlite3_exec(db, sql, callback, 0, &zErrMsg);
+    if( rc != SQLITE_OK ){
+        fprintf(stderr, "SQL error: %s\n", zErrMsg);
+        sqlite3_free(zErrMsg);
+        sqlite3_close(db);
+        return 0;
+    }
+    const char* zSql = "INSERT INTO connections (starttime,endtime,src_ipn,dst_ipn,mac_daddr,mac_saddr,packets,srcport,dstport,hashdigest_md5) VALUES (?,?,?,?,?,?,?,?,?,?)";
+    if(sqlite3_prepare_v2(db, zSql, strlen(zSql), &insert_stmt, NULL)!=SQLITE_OK ){
+        fprintf(stderr, "SQL prepare error");
+        db = 0;
+        insert_stmt=0;
+        return(0);
+    }
+#endif
+}
+
+void  tcpdemux::write_flow_record(const std::string &starttime,const std::string &endtime,
+                        const std::string &src_ipn,const std::string &dst_ipn,
+                        const std::string &mac_daddr,const std::string &mac_saddr,
+                        uint64_t packets,uint16_t srcport,uint16_t dstport,
+                        const std::string &hashdigest_md5)
+{
+}
+
+
+
 /* static */ tcpdemux *tcpdemux::getInstance()
 {
     static tcpdemux * theInstance = 0;
@@ -75,7 +125,8 @@ void tcpdemux::close_oldest_fd()
 int tcpdemux::retrying_open(const std::string &filename,int oflag,int mask)
 {
     while(true){
-	if(open_flows.size() >= max_fds) close_oldest_fd();
+    //Packet index file reduces max_fds by 1/2 as the index files also take a fd
+	if(open_flows.size() >= (opt.output_packet_index ?  max_fds/2 : max_fds)) close_oldest_fd();
 	int fd = ::open(filename.c_str(),oflag,mask);
 	DEBUG(2)("retrying_open ::open(fn=%s,oflag=x%x,mask:x%x)=%d",filename.c_str(),oflag,mask,fd);
 	if(fd>=0){
@@ -127,15 +178,26 @@ tcpip *tcpdemux::create_tcpip(const flow_addr &flowa, be13::tcp_seq isn,const be
 
     tcpip *new_tcpip = new tcpip(*this,flow,isn);
     new_tcpip->nsn   = isn+1;		// expected sequence number of the first byte
-    DEBUG(5) ("%s: new flow. next seq num (nsn):%d", new_tcpip->flow_pathname.c_str(),new_tcpip->nsn);
+    DEBUG(5) ("new flow %s. path: %s next seq num (nsn):%d",
+              flowa.str().c_str(),new_tcpip->flow_pathname.c_str(),new_tcpip->nsn);
     flow_map[flow] = new_tcpip;
     return new_tcpip;
 }
 
 /**
- * remove a flow from the database and close the flow
- * These are the only places where a tcpip object is deleted so there is no chance of finding it again.
- * This typically happens at the end of the run unless items in the map were removed.
+ * Remove a flow from the database.
+ * Close the flow file.
+ * Write to the report.xml object.
+ * Save in the sqlite database.
+ * This is the ONLY place where a tcpip object is deleted so there is no chance of finding it again.
+ * 
+ * Flows are post-processed when a FIN is received and all bytes are received.
+ * If a FIN is received and bytes are outstanding, they are post-processed when the last byte is received.
+ * When the program shut down, all open flows are post-processed.
+ * 
+ * Amended to trigger the packet/data location index sort as part of the post-processing.  This sorts
+ * the (potentially out of order) index to make it simple for external applications.  No processing is
+ * done if the (-I) index generation feature is turned off.  --GDD
  */
 
 void tcpdemux::post_process(tcpip *tcp)
@@ -152,7 +214,7 @@ void tcpdemux::post_process(tcpip *tcp)
         /* Open the fd if it is not already open */
         tcp->open_file();
         if(tcp->fd>=0){
-            sbuf_t *sbuf = sbuf_t::map_file(tcp->flow_pathname,pos0_t(tcp->flow_pathname),tcp->fd);
+            sbuf_t *sbuf = sbuf_t::map_file(tcp->flow_pathname,tcp->fd);
             if(sbuf){
                 be13::plugin::process_sbuf(scanner_params(scanner_params::PHASE_SCAN,*sbuf,*(fs),&xmladd));
                 delete sbuf;
@@ -292,6 +354,8 @@ void tcpdemux::save_flow(tcpip *tcp)
  * 
  * creates a new tcp connection if necessary, then asks the connection to either
  * print the packet or store it.
+ *
+ * Returns 0 if packet is processed, 1 if it is not processed, -1 if error
  */
 
 #define FLAG_SET(vector, flag) ((vector) & (flag))
@@ -306,7 +370,7 @@ int tcpdemux::process_tcp(const ipaddr &src, const ipaddr &dst,sa_family_t famil
     if (ip_payload_len < sizeof(struct be13::tcphdr)) {
 	DEBUG(6) ("received truncated TCP segment! (%u<%u)",
                   (u_int)ip_payload_len,(u_int)sizeof(struct be13::tcphdr));
-	return 0;
+	return 1;
     }
 
     struct be13::tcphdr *tcp_header = (struct be13::tcphdr *) ip_data;
@@ -318,6 +382,7 @@ int tcpdemux::process_tcp(const ipaddr &src, const ipaddr &dst,sa_family_t famil
     bool syn_set = FLAG_SET(tcp_header->th_flags, TH_SYN);
     bool ack_set = FLAG_SET(tcp_header->th_flags, TH_ACK);
     bool fin_set = FLAG_SET(tcp_header->th_flags, TH_FIN);
+    bool rst_set = FLAG_SET(tcp_header->th_flags, TH_RST);
 
     /* calculate the total length of the TCP header including options */
     u_int tcp_header_len = tcp_header->th_off * 4;
@@ -334,12 +399,21 @@ int tcpdemux::process_tcp(const ipaddr &src, const ipaddr &dst,sa_family_t famil
     int32_t  delta = 0;			// from current position in tcp connection; must be SIGNED 32 bit!
     tcpip   *tcp = find_tcpip(this_flow);
     
+    DEBUG(60)("%s%s%s%s tcp_header_len=%d tcp_datalen=%d seq=%u tcp=%p",
+              (syn_set?"SYN ":""),(ack_set?"ACK ":""),(fin_set?"FIN ":""),(rst_set?"RST ":""),(int)tcp_header_len,(int)tcp_datalen,(int)seq,tcp);
+
     /* If this_flow is not in the database and the start_new_connections flag is false, just return */
     if(tcp==0 && start_new_connections==false) return 0; 
 
+    if(syn_set && tcp && tcp->syn_count>0 && tcp->pos>0){
+        std::cerr << "SYN TO IGNORE! SYN tcp="<<tcp << " flow="<<this_flow<<"\n";
+        return 1;
+    }
+
     if(tcp==0){
         if(tcp_datalen==0){                       // zero length packet
             if(fin_set) return 0;              // FIN on a connection that's unknown; safe to ignore
+            if(rst_set) return 0;              // RST on a connection that's unknown; safe to ignore
             if(syn_set==false && ack_set==false) return 0; // neither a SYN nor ACK; return
         } else {
             /* Data present on a flow that is not actively being demultiplexed.
@@ -355,7 +429,7 @@ int tcpdemux::process_tcp(const ipaddr &src, const ipaddr &dst,sa_family_t famil
                 if(fd>0){
                     char *buf = (char *)malloc(tcp_datalen);
                     if(buf){
-                        DEBUG(100)("lseek(fd,%"PRId64",SEEK_SET)",(int64_t)(offset));
+                        DEBUG(100)("lseek(fd,%" PRId64 ",SEEK_SET)",(int64_t)(offset));
                         lseek(fd,offset,SEEK_SET);
                         ssize_t r = read(fd,buf,tcp_datalen);
                         data_match = (r==(ssize_t)tcp_datalen) && memcmp(buf,tcp_data,tcp_datalen)==0;
@@ -460,11 +534,16 @@ int tcpdemux::process_tcp(const ipaddr &src, const ipaddr &dst,sa_family_t famil
 	    tcp->print_packet(tcp_data, tcp_datalen);
 	} else {
 	    if (opt.store_output){
-		tcp->store_packet(tcp_data, tcp_datalen, delta);
+		tcp->store_packet(tcp_data, tcp_datalen, delta,pi.ts);
 	    }
 	}
     }
 
+    if (rst_set){
+        remove_flow(this_flow);	// take it out of the map  
+        return 0;
+    }
+
     /* Count the FINs.
      * If this is a fin, determine the size of the stream
      */
@@ -599,6 +678,7 @@ int tcpdemux::process_ip6(const be13::packet_info &pi)
 #pragma GCC diagnostic ignored "-Wcast-align"
 int tcpdemux::process_pkt(const be13::packet_info &pi)
 {
+    DEBUG(10)("process_pkt..............................................................................");
     int r = 1;                          // not processed yet
     switch(pi.ip_version()){
     case 4:
@@ -615,17 +695,19 @@ int tcpdemux::process_pkt(const be13::packet_info &pi)
 
     /* Process the timeout, if there is any */
     if(tcp_timeout){
-        std::vector<flow *> to_close;
+        /* Get a list of the flows that need to be closed.  */
+        std::vector<flow_addr *> to_close;
         for(flow_map_t::iterator it = flow_map.begin(); it!=flow_map.end(); it++){
             tcpip &tcp = *(it->second);
-            flow &f = tcp.myflow;
-            uint32_t age = pi.ts.tv_sec - f.tlast.tv_sec;
+            uint32_t age = pi.ts.tv_sec - tcp.myflow.tlast.tv_sec;
             if (age > tcp_timeout){
-                to_close.push_back(&f);
+                to_close.push_back(&tcp.myflow);
             }
         }
-
-        for(std::vector<flow *>::iterator it = to_close.begin(); it!=to_close.end(); it++){
+        /* Close them. This removes the flows from the flow_map(), which is why we need
+         * to create the list first.
+         */
+        for(std::vector<flow_addr *>::iterator it = to_close.begin(); it!=to_close.end(); it++){
             remove_flow(*(*it));
         }
     }
