@@ -25,9 +25,6 @@
 #include <sys/types.h>
 #include <dirent.h>
 
-
-
-
 /* bring in inet_ntop if it is not present */
 #define ETH_ALEN 6
 #ifndef HAVE_INET_NTOP
@@ -72,7 +69,9 @@ scanner_t *scanners_builtin[] = {
     scan_http,
     scan_netviz,
     scan_tcpdemux,
+#ifdef USE_WIFI
     scan_wifiviz,
+#endif
     0};
 
 bool opt_no_promisc = false;		// true if we should not use promiscious mode
@@ -81,84 +80,80 @@ bool opt_no_promisc = false;		// true if we should not use promiscious mode
  *** USAGE
  ****************************************************************/
 
-static int usage_count = 0;
-static void usage()
+static void usage(int level)
 {
-    switch(++usage_count){
-    case 1:
-        std::cout << PACKAGE_NAME << " version " << PACKAGE_VERSION << "\n\n";
-        std::cout << "usage: " << progname << " [-aBcCDhJpsvVZ] [-b max_bytes] [-d debug_level] \n";
-        std::cout << "     [-[eE] scanner] [-f max_fds] [-F[ctTXMkmg]] [-i iface] [-L semlock]\n";
-        std::cout << "     [-m min_bytes] [-o outdir] [-r file] [-R file] \n";
-        std::cout << "     [-S name=value] [-T template] [-w file] [-x scanner] [-X xmlfile]\n";
-        std::cout << "      [expression]\n\n";
-        std::cout << "   -a: do ALL post-processing.\n";
-        std::cout << "   -b max_bytes: max number of bytes per flow to save\n";
-        std::cout << "   -d debug_level: debug level; default is " << DEFAULT_DEBUG_LEVEL << "\n";
-        std::cout << "   -f: maximum number of file descriptors to use\n";
-        std::cout << "   -h: print this help message (-hh for more help)\n";
-        std::cout << "   -H: print detailed information about each scanner\n";
-        std::cout << "   -i: network interface on which to listen\n";
-        std::cout << "   -J: output each flow in alternating colors (note change!)\n";
-        std::cout << "   -l: treat non-flag arguments as input files rather than a pcap expression\n";
-        std::cout << "   -L  semlock - specifies that writes are locked using a named semaphore\n";
-        std::cout << "   -p: don't use promiscuous mode\n";
-        std::cout << "   -q: quiet mode - do not print warnings\n";
-        std::cout << "   -r file: read packets from tcpdump pcap file (may be repeated)\n";
-        std::cout << "   -R file: read packets from tcpdump pcap file TO FINISH CONNECTIONS\n";
-        std::cout << "   -v: verbose operation equivalent to -d 10\n";
-        std::cout << "   -V: print version number and exit\n";
-        std::cout << "   -w file: write packets not processed to file\n";
-        std::cout << "   -o  outdir   : specify output directory (default '.')\n";
-        std::cout << "   -X  filename : DFXML output to filename\n";
-        std::cout << "   -m  bytes    : specifies skip that starts a new stream (default "
-                  << (unsigned)tcpdemux::options::MAX_SEEK << ").\n";
-        std::cout << "   -F{p} : filename prefix/suffix (-hh for options)\n";
-        std::cout << "   -T{t} : filename template (-hh for options; default "
-                  << flow::filename_template << ")\n";
-        std::cout << "   -Z: do not decompress gzip-compressed HTTP transactions\n";
+    std::cout << PACKAGE_NAME << " version " << PACKAGE_VERSION << "\n\n";
+    std::cout << "usage: " << progname << " [-aBcCDhJpsvVZ] [-b max_bytes] [-d debug_level] \n";
+    std::cout << "     [-[eE] scanner] [-f max_fds] [-F[ctTXMkmg]] [-i iface] [-L semlock]\n";
+    std::cout << "     [-m min_bytes] [-o outdir] [-r file] [-R file] \n";
+    std::cout << "     [-S name=value] [-T template] [-w file] [-x scanner] [-X xmlfile]\n";
+    std::cout << "      [expression]\n\n";
+    std::cout << "   -a: do ALL post-processing.\n";
+    std::cout << "   -b max_bytes: max number of bytes per flow to save\n";
+    std::cout << "   -d debug_level: debug level; default is " << DEFAULT_DEBUG_LEVEL << "\n";
+    std::cout << "   -f: maximum number of file descriptors to use\n";
+    std::cout << "   -h: print this help message (-hh for more help)\n";
+    std::cout << "   -H: print detailed information about each scanner\n";
+    std::cout << "   -i: network interface on which to listen\n";
+    std::cout << "   -I: generate temporal packet-> byte index files for each flow (.findex)\n";
+    std::cout << "   -g: output each flow in alternating colors (note change!)\n";
+    std::cout << "   -l: treat non-flag arguments as input files rather than a pcap expression\n";
+    std::cout << "   -L  semlock - specifies that writes are locked using a named semaphore\n";
+    std::cout << "   -p: don't use promiscuous mode\n";
+    std::cout << "   -q: quiet mode - do not print warnings\n";
+    std::cout << "   -r file: read packets from tcpdump pcap file (may be repeated)\n";
+    std::cout << "   -R file: read packets from tcpdump pcap file TO FINISH CONNECTIONS\n";
+    std::cout << "   -v: verbose operation equivalent to -d 10\n";
+    std::cout << "   -V: print version number and exit\n";
+    std::cout << "   -w file: write packets not processed to file\n";
+    std::cout << "   -o  outdir   : specify output directory (default '.')\n";
+    std::cout << "   -X  filename : DFXML output to filename\n";
+    std::cout << "   -m  bytes    : specifies skip that starts a new stream (default "
+              << (unsigned)tcpdemux::options::MAX_SEEK << ").\n";
+    std::cout << "   -F{p} : filename prefix/suffix (-hh for options)\n";
+    std::cout << "   -T{t} : filename template (-hh for options; default "
+              << flow::filename_template << ")\n";
+    std::cout << "   -Z: do not decompress gzip-compressed HTTP transactions\n";
 
-        std::cout << "\nControl of Scanners:\n";
-        std::cout << "   -E scanner   - turn off all scanners except scanner\n";
-        std::cout << "   -S name=value  Set a configuration parameter (-hh for info)\n";
-        be13::plugin::info_scanners(false,true,scanners_builtin,'e','x');
+    std::cout << "\nControl of Scanners:\n";
+    std::cout << "   -E scanner   - turn off all scanners except scanner\n";
+    std::cout << "   -S name=value  Set a configuration parameter (-hh for info)\n";
+    be13::plugin::info_scanners(false,true,scanners_builtin,'e','x');
 
-        std::cout << "Console output options:\n";
-        std::cout << "   -B: binary output, even with -c or -C (normally -c or -C turn it off)\n";
-        std::cout << "   -c: console print only (don't create files)\n";
-        std::cout << "   -C: console print only, but without the display of source/dest header\n";
-        std::cout << "   -s: strip non-printable characters (change to '.')\n";
-        std::cout << "   -D: output in hex (useful to combine with -c or -C)\n";
-        std::cout << "\n";
+    std::cout << "Console output options:\n";
+    std::cout << "   -B: binary output, even with -c or -C (normally -c or -C turn it off)\n";
+    std::cout << "   -c: console print only (don't create files)\n";
+    std::cout << "   -C: console print only, but without the display of source/dest header\n";
+    std::cout << "   -0: don't print newlines after packets when printing to console";
+    std::cout << "   -s: strip non-printable characters (change to '.')\n";
+    std::cout << "   -D: output in hex (useful to combine with -c or -C)\n";
+    std::cout << "\n";
 #ifndef HAVE_LIBCAIRO
-	std::cout << "Rendering not available because Cairo was not installed.\n\n";
+    std::cout << "Rendering not available because Cairo was not installed.\n\n";
 #endif
-        std::cout << "expression: tcpdump-like filtering expression\n";
-        std::cout << "\nSee the man page for additional information.\n\n";
-        break;
-    case 2:
-        std::cout << "Filename Prefixes:\n";
-        std::cout << "   -Fc : append the connection counter to ALL filenames\n";
-        std::cout << "   -Ft : prepend the time_t timestamp to ALL filenames\n";
-        std::cout << "   -FT : prepend the ISO8601 timestamp to ALL filenames\n";
-        std::cout << "   -FX : Do not output any files (other than report files)\n";
-        std::cout << "   -FM : Calculate the MD5 for every flow (stores in DFXML)\n";
-        std::cout << "   -Fk : Bin output in 1K directories\n";
-        std::cout << "   -Fm : Bin output in 1M directories (2 levels)\n";
-        std::cout << "   -Fg : Bin output in 1G directories (3 levels)\n";
-        flow::usage();
-        std::cout << "-S name=value options:\n";
-        for(int i=0;defaults[i].name;i++){
-            std::stringstream ss;
-            ss << defaults[i].name << "=" << defaults[i].dvalue;
-            printf("   %-20s %s\n",ss.str().c_str(),defaults[i].help);
-        }
-        std::cout << "\n";
-        std::cout << "DEBUG Levels (specify with -dNN):\n";
-        std::cout << "get_max_fds() = " << tcpdemux::getInstance()->get_max_fds() << "\n";
-        std::cout << "NUM_RESERVED_FDS = " << NUM_RESERVED_FDS << "\n";
-        break;
+    std::cout << "expression: tcpdump-like filtering expression\n";
+    std::cout << "\nSee the man page for additional information.\n\n";
+    if(level<2) return;
+    std::cout << "Filename Prefixes:\n";
+    std::cout << "   -Fc : append the connection counter to ALL filenames\n";
+    std::cout << "   -Ft : prepend the time_t timestamp to ALL filenames\n";
+    std::cout << "   -FT : prepend the ISO8601 timestamp to ALL filenames\n";
+    std::cout << "   -FX : Do not output any files (other than report files)\n";
+    std::cout << "   -FM : Calculate the MD5 for every flow (stores in DFXML)\n";
+    std::cout << "   -Fk : Bin output in 1K directories\n";
+    std::cout << "   -Fm : Bin output in 1M directories (2 levels)\n";
+    std::cout << "   -Fg : Bin output in 1G directories (3 levels)\n";
+    flow::usage();
+    std::cout << "-S name=value options:\n";
+    for(int i=0;defaults[i].name;i++){
+        std::stringstream ss;
+        ss << defaults[i].name << "=" << defaults[i].dvalue;
+        printf("   %-20s %s\n",ss.str().c_str(),defaults[i].help);
     }
+    std::cout << "\n";
+    std::cout << "DEBUG Levels (specify with -dNN):\n";
+    std::cout << "get_max_fds() = " << tcpdemux::getInstance()->get_max_fds() << "\n";
+    std::cout << "NUM_RESERVED_FDS = " << NUM_RESERVED_FDS << "\n";
 }
 
 /**
@@ -283,7 +278,9 @@ static inflaters_t *build_inflaters()
  * May be repeated.
  * If start is false, do not initiate new connections
  */
+#ifdef HAVE_INFLATER
 static inflaters_t *inflaters = 0;
+#endif
 static void process_infile(const std::string &expression,const char *device,const std::string &infile)
 {
     char error[PCAP_ERRBUF_SIZE];
@@ -291,7 +288,9 @@ static void process_infile(const std::string &expression,const char *device,cons
     int dlt=0;
     pcap_handler handler;
 
+#ifdef HAVE_INFLATER
     if(inflaters==0) inflaters = build_inflaters();
+#endif
 
     if (infile!=""){
         std::string file_path = infile;
@@ -373,11 +372,32 @@ static void process_infile(const std::string &expression,const char *device,cons
 }
 
 
+/* be_hash. Currently this just returns the MD5 of the sbuf,
+ * but eventually it will allow the use of different hashes.
+ */
+static std::string be_hash_name("md5");
+static std::string be_hash_func(const uint8_t *buf,size_t bufsize)
+{
+    if(be_hash_name=="md5" || be_hash_name=="MD5"){
+        return md5_generator::hash_buf(buf,bufsize).hexdigest();
+    }
+    if(be_hash_name=="sha1" || be_hash_name=="SHA1" || be_hash_name=="sha-1" || be_hash_name=="SHA-1"){
+        return sha1_generator::hash_buf(buf,bufsize).hexdigest();
+    }
+    if(be_hash_name=="sha256" || be_hash_name=="SHA256" || be_hash_name=="sha-256" || be_hash_name=="SHA-256"){
+        return sha256_generator::hash_buf(buf,bufsize).hexdigest();
+    }
+    std::cerr << "Invalid hash name: " << be_hash_name << "\n";
+    std::cerr << "This version of bulk_extractor only supports MD5, SHA1, and SHA256\n";
+    exit(1);
+}
+static feature_recorder_set::hash_def be_hash(be_hash_name,be_hash_func);
 
 
 int main(int argc, char *argv[])
 {
-    bool didhelp = false;
+    int opt_help = 0;
+    int opt_Help = 0;
     feature_recorder::set_main_threadid();
     sbuf_t::set_map_file_delimiter(""); // no delimiter on carving
 #ifdef BROKEN
@@ -387,8 +407,9 @@ int main(int argc, char *argv[])
     std::cerr << "\n";
 #endif
 
+    bool opt_enable_report = true;
     bool force_binary_output = false;
-    const char *device = "<default>";
+    const char *device = 0;             // default device
     const char *lockname = 0;
     int need_usage = 0;
     std::string reportfilename;
@@ -414,7 +435,7 @@ int main(int argc, char *argv[])
 
     bool trailing_input_list = false;
     int arg;
-    while ((arg = getopt(argc, argv, "aA:Bb:cCd:DE:e:E:F:f:Hhi:JlL:m:o:pqR:r:S:sT:Vvw:x:X:Z")) != EOF) {
+    while ((arg = getopt(argc, argv, "aA:Bb:cCd:DE:e:E:F:f:gHhIi:lL:m:o:pqR:r:S:sT:Vvw:x:X:Z:0")) != EOF) {
 	switch (arg) {
 	case 'a':
 	    demux.opt.post_processing = true;
@@ -444,6 +465,9 @@ int main(int argc, char *argv[])
 	case 'c':
 	    demux.opt.console_output = true;	DEBUG(10) ("printing packets to console only");
 	    break;
+    case '0':
+	    demux.opt.console_output_nonewline = true;
+	    break;
 	case 'd':
 	    if ((debug = atoi(optarg)) < 0) {
 		debug = DEFAULT_DEBUG_LEVEL;
@@ -487,7 +511,11 @@ int main(int argc, char *argv[])
 	    break;
         }
 	case 'i': device = optarg; break;
-	case 'J':
+ 	case 'I':
+ 		DEBUG(10) ("creating packet index files");
+ 		demux.opt.output_packet_index = true;
+ 		break;
+	case 'g':
 	    demux.opt.use_color  = 1;
 	    DEBUG(10) ("using colors");
 	    break;
@@ -530,33 +558,44 @@ int main(int argc, char *argv[])
 	case 'x': be13::plugin::scanners_disable(optarg);break;
 	case 'X': reportfilename = optarg;break;
 	case 'Z': demux.opt.gzip_decompress = 0; break;
-	case 'H':
-            be13::plugin::info_scanners(true,true,scanners_builtin,'e','x');
-            didhelp = true;
-            break;
-	case 'h': case '?':
-            usage();
-            didhelp = true;
-            break;
+	case 'H': opt_Help += 1; break;
+	case 'h': opt_help += 1; break;
 	default:
 	    DEBUG(1) ("error: unrecognized switch '%c'", arg);
-	    need_usage = 1;
+	    opt_help += 1;
 	    break;
 	}
     }
 
-    if(didhelp) exit(0);
+    
+    argc -= optind;
+    argv += optind;
+
+
+    /* Load all the scanners and enable the ones we care about */
+    scanner_info si;
+    si.config = &be_config;
+
+    si.get_config("enable_report",&opt_enable_report,"Enable report.xml");
+    be13::plugin::load_scanners(scanners_builtin,be_config);
+
+    if(opt_Help){
+        be13::plugin::info_scanners(true,true,scanners_builtin,'e','x');
+        exit(0);
+    }
+
+    if(opt_help) {
+        usage(opt_help);
+        exit(0);
+    }
+            
+
     if(demux.opt.post_processing && !demux.opt.store_output){
         std::cerr << "ERROR: post_processing currently requires storing output.\n";
         exit(1);
     }
 
-    argc -= optind;
-    argv += optind;
-
-    /* Load all the scanners and enable the ones we care about */
     if(demux.opt.opt_md5) be13::plugin::scanners_enable("md5");
-    be13::plugin::load_scanners(scanners_builtin,be_config);
     be13::plugin::scanners_process_enable_disable_commands();
 
     /* If there is no report filename, call it report.xml in the output directory */
@@ -564,12 +603,6 @@ int main(int argc, char *argv[])
 	reportfilename = demux.outdir + "/" + DEFAULT_REPORT_FILENAME;
     }
 
-    /* print help and exit if there was an error in the arguments */
-    if (need_usage) {
-	usage();
-	exit(1);
-    }
-
     /* remaining arguments are either an input list (-l flag) or a pcap expression (default) */
     std::string expression = "";
     if(trailing_input_list) {
@@ -621,7 +654,7 @@ int main(int argc, char *argv[])
     }
 
     /* report file specified? */
-    if(reportfilename.size()>0){
+    if(reportfilename.size()>0 && opt_enable_report){
 	xreport = new dfxml_writer(reportfilename,false);
 	dfxml_create(*xreport,command_line);
 	demux.xreport = xreport;
@@ -635,8 +668,6 @@ int main(int argc, char *argv[])
         demux.save_unk_packets(opt_unk_packets,input_fname);
     }
 
-    scanner_info si;
-    si.config = &be_config;
 
     /* Debug prefix set? */
     std::string debug_prefix=progname;
@@ -645,11 +676,14 @@ int main(int argc, char *argv[])
 
     DEBUG(10) ("%s version %s ", PACKAGE_NAME, PACKAGE_VERSION);
 
+    const char *name = device;
+    if(input_fname.size()>0) name=input_fname.c_str();
+    if(name==0) name="<default>";
+
     feature_file_names_t feature_file_names;
     be13::plugin::get_scanner_feature_file_names(feature_file_names);
-    feature_recorder_set fs(0);
-
-    fs.init(feature_file_names,input_fname.size()>0 ? input_fname : device,demux.outdir);
+    feature_recorder_set fs(feature_recorder_set::NO_ALERT,be_hash,name,demux.outdir);
+    fs.init(feature_file_names);
     the_fs   = &fs;
     demux.fs = &fs;
 
